@@ -11,10 +11,12 @@ import Common.Annex
 import Command
 import Upgrade
 import Annex.Version
+import Config
 
 def :: [Command]
 def = [dontCheck repoExists $ -- because an old version may not seem to exist
-	command "upgrade" paramNothing seek "upgrade repository layout"]
+	command "upgrade" paramNothing seek
+		SectionMaintenance "upgrade repository layout"]
 
 seek :: [CommandSeek]
 seek = [withNothing start]
@@ -22,6 +24,9 @@ seek = [withNothing start]
 start :: CommandStart
 start = do
 	showStart "upgrade" "."
-	r <- upgrade
-	setVersion
+	r <- upgrade False
+	ifM isDirect
+		( setVersion directModeVersion
+		, setVersion defaultVersion
+		)
 	next $ next $ return r
