@@ -12,8 +12,9 @@ module Git.Repair (
 	retrieveMissingObjects,
 	resetLocalBranches,
 	removeTrackingBranches,
-	rewriteIndex,
 	checkIndex,
+	missingIndex,
+	nukeIndex,
 	emptyGoodCommits,
 ) where
 
@@ -368,7 +369,9 @@ verifyTree missing treesha r
 			-- as long as ls-tree succeeded, we're good
 			else cleanup
 
-{- Checks that the index file only refers to objects that are not missing. -}
+{- Checks that the index file only refers to objects that are not missing,
+ - and is not itself corrupt. Note that a missing index file is not
+ - considered a problem (repo may be new). -}
 checkIndex :: MissingObjects -> Repo -> IO Bool
 checkIndex missing r = do
 	(bad, _good, cleanup) <- partitionIndex missing r
@@ -378,6 +381,9 @@ checkIndex missing r = do
 			void cleanup
 			return False
 
+missingIndex :: Repo -> IO Bool
+missingIndex r = not <$> doesFileExist (localGitDir r </> "index")
+
 partitionIndex :: MissingObjects -> Repo -> IO ([LsFiles.StagedDetails], [LsFiles.StagedDetails], IO Bool)
 partitionIndex missing r = do
 	(indexcontents, cleanup) <- LsFiles.stagedDetails [repoPath r] r
@@ -396,7 +402,7 @@ rewriteIndex missing r
 	| otherwise = do
 		(bad, good, cleanup) <- partitionIndex missing r
 		unless (null bad) $ do
-			nukeFile (localGitDir r </> "index")
+			nukeIndex r
 			UpdateIndex.streamUpdateIndex r
 				=<< (catMaybes <$> mapM reinject good)
 		void cleanup
@@ -408,6 +414,9 @@ rewriteIndex missing r
 			UpdateIndex.stageFile sha blobtype file r
 	reinject _ = return Nothing
 
+nukeIndex :: Repo -> IO ()
+nukeIndex r = nukeFile (localGitDir r </> "index")
+
 newtype GoodCommits = GoodCommits (S.Set Sha)
 
 emptyGoodCommits :: GoodCommits
@@ -448,7 +457,16 @@ runRepairOf fsckresult forced referencerepo g = do
 	missing <- cleanCorruptObjects fsckresult g
 	stillmissing <- retrieveMissingObjects missing referencerepo g
 	if S.null stillmissing
-		then successfulfinish stillmissing []
+		then if repoIsLocalBare g
+			then successfulfinish stillmissing []
+			else ifM (checkIndex stillmissing g)
+				( successfulfinish stillmissing []
+				, do
+					putStrLn "No missing objects found, but the index file is corrupt!"
+					if forced
+						then corruptedindex
+						else needforce stillmissing
+				)		
 		else do
 			putStrLn $ unwords
 				[ show (S.size stillmissing)
@@ -491,6 +509,12 @@ runRepairOf fsckresult forced referencerepo g = do
 				putStrLn "Successfully recovered repository!"
 				putStrLn "Please carefully check that the changes mentioned above are ok.."
 				return (True, stillmissing, modifiedbranches)
+	
+	corruptedindex = do
+		nukeIndex g
+		putStrLn "Removed the corrupted index file. You should look at what files are present in your working tree and git add them back to the index when appropriate."
+		return (True, S.empty, [])
+
 	successfulfinish stillmissing modifiedbranches = do
 		mapM_ putStrLn
 			[ "Successfully recovered repository!"
@@ -503,5 +527,8 @@ runRepairOf fsckresult forced referencerepo g = do
 			then do
 				putStrLn "If you have a clone of this bare repository, you should add it as a remote of this repository, and re-run git-recover-repository."
 				putStrLn "If there are no clones of this repository, you can instead run git-recover-repository with the --force parameter to force recovery to a possibly usable state."
-			else putStrLn "To force a recovery to a usable state, run this command again with the --force parameter."
+				return (False, stillmissing, [])
+			else needforce stillmissing
+	needforce stillmissing = do
+		putStrLn "To force a recovery to a usable state, run this command again with the --force parameter."
 		return (False, stillmissing, [])
